/**
 * NORRIS-IoC.js
 * Copyright (c) 2011 Ruben L Z Tan
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a 
 * copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS 
 * IN THE SOFTWARE.
 */
'use strict';

///////////////////////////////////////////////////////////////////////////////
// @nordoc[package] ioc.installer
///////////////////////////////////////////////////////////////////////////////
module.exports.make = function make(deps) {
    var base, npm, fs, installer;

    base    = deps.base,
    npm     = deps.npm,
    fs      = deps.fs;

    ///////////////////////////////////////////////////////////////////////////
    // @nordoc[module] installer
    ///////////////////////////////////////////////////////////////////////////
    installer = {

        //---------------------------------------------------------------------
        // @nordoc[method]
        //
        // @desc    Installs the NPM module
        //
        // @args    module [string]
        //          The name of the module to install
        //
        //          cb [function] (err, module)
        //          The callback to call upon completion of the installation
        //          process
        //
        // @return  Returns itself
        //---------------------------------------------------------------------
        install : function install(module, cb) {
            npm.prefix = base;

            npm.commands.install([module], function onInstall(err, data) {
                var modpath, meta, npmMod, installPath;

                if (err) {
                    cb(err);
                    return;
                }

                try {
                    installPath = (data.shift())[1];

                    modpath = base + '/node_modules/' + module;
                    meta    = JSON.parse(fs.readFileSync(__dirname + '/' + installPath + '/package.json', 'utf-8'));

                    if (undefined === meta.main) {
                        cb(new Error('No "main" file defined. I don\'t know how to load this'));
                        return;
                    }
                    
                    //npmMod = require(modpath + '/' + meta.main);
                    npmMod = require(module);
                    cb(null, npmMod);
                } catch (err) {
                    cb(err);
                }
            });

            return this;
        }
    }

    return installer;
}